/*******************************************************************************
 * IBM Confidential
 * OCO Source Materials
 * IBM HTML Commons library Version 0.01 (IBM-HTML-Commons-0001)
 * (c) Copyright IBM Corporation 2006.
 * All rights reserved.
 * 
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been 
 * deposited with the U.S. Copyright Office.
 * 
 * IBM Confidential
 *     2005/11/7 - updated by Mei Yang (IBM Corp.) for Quick Print uat4i00000015
 *******************************************************************************/

package com.ibm.watson.commons.html.parseutils;

/**
 * An abstract class for results of recursive descent parsing functions. They
 * contain the result of parsing a stream of tokens to identify an instance of
 * type <code>T</code>. The parsing results should include the identified
 * instance of type <code>T</code> (available through the method
 * <code>getParsedValue</code>) and the remainder of the stream left to be
 * parsed (available through the method <code>getRemainder</code>).
 * 
 * See the classes <code>WebSearchEngine</code> and
 * <code>RobustHTMLParser</code> for examples of the use of these classes.
 * 
 * @author Pablo A. Duboue (<a
 *         href="mailto:duboue@us.ibm.com">duboue@us.ibm.com</a>)
 */
public abstract class ParseResult {
	 private static final String copyright = "Licensed Materials - Property of IBM\n"
         + "IEHS-311\n"
         + "(c) Copyright IBM Corporation 2006.\n"
         + "All rights reserved.\n"
         + "US Government Users Restricted Right - Use, duplication or\n"
         + "disclosure restricted by GSA ADP Schedule Contract with\n"
         + "IBM Corporation.";

	protected Integer remainder;

	protected int remainderAsInt = 0;

	protected ParseResult(Integer remainder) {
		this.remainder = remainder;
		if (remainder != null)
			this.remainderAsInt = remainder.intValue();
	}

	/**
	 * @return the parsed value, if any. Null otherwise.
	 */
	public abstract Object getParsedValue();

	/**
	 * @return the remainder of the stream after the current item has been
	 *         parsed. This type can be the stream itself or a position in an
	 *         underlining stream. In case of an error, it is the position a
	 *         continuation of the parse can be attempted, if available by the
	 *         underlining grammar.
	 */
	public Integer getRemainder() {
		return this.remainder;
	}

	public int getRemainderAsInt() {
		return this.remainderAsInt;
	}
}
