/*******************************************************************************
* IBM Confidential
 * OCO Source Materials
 * IEHS-3.4
 * (c) Copyright IBM Corporation 2008.
 * All rights reserved.
 * 
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been 
 * deposited with the U.S. Copyright Office.
*******************************************************************************/


package org.eclipse.help.internal.webapp.data;

import java.util.ArrayList;
import java.util.BitSet;
import java.util.Collection;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.eclipse.help.IFilterableResource;
import org.eclipse.help.internal.HelpPlugin;
import org.eclipse.help.internal.base.HelpBasePlugin;
import org.eclipse.help.internal.toc.FilterManager;

/**
 * this class is use to init the criteria and filter topics
 * 
 * @author hongg
 * 
 */
public class FilterData {

	/**
	 * criterias from toc.xml
	 */
	private Map<String,List<String>> criteria;
	private String locale;
	HttpServletRequest request = null;
	/**
	 * the current session
	 */
	private HttpSession session;

	private boolean cookiEnabled = true;

	/**
	 * only init the session it is only for the filter method
	 * 
	 * @param session
	 *            the current session
	 */
	private FilterData(HttpSession session) {
		this.session = session;
		criteria = (Map) session.getAttribute("allCriterias");
	}

	/**
	 * init the session and parse user request to init the user filter
	 * 
	 * @param request
	 *            the current user request
	 * @param session
	 *            the current session
	 */
	public FilterData(HttpServletRequest request, HttpServletResponse response,
			ServletContext context) {
		this.session = request.getSession(); //filter bug
		this.request = request;
		// initial the criteria by FilterManager or set it as an empty hashMap
		locale = UrlUtil.getLocale(request, response);
		FilterManager fm = HelpPlugin.getFilterManager();
		criteria = fm.getCriterias(locale);
		if(null == criteria){
			criteria = Collections.emptyMap();
		}
		if (session.getAttribute("filterTag") == null
				|| session.getAttribute("allCriterias") != criteria) {
			session.setAttribute(CRITERIA, fm.refineSessionCriteria(cloneMap(criteria),locale));
			session.setAttribute("allCriterias", criteria);
			session.setAttribute("filterTag", "filterTag");
		}
		// parse params from session
		Enumeration e = request.getParameterNames();
		if (e.hasMoreElements()) {
			String disableFilter = (String) request
					.getParameter(DISABLE_FILTER);
			if (FALSE.equals(disableFilter)) { // user enabled the filter

				String noCriteria = request.getParameter(NOCRITERIA);
				if(null != noCriteria && Boolean.valueOf(noCriteria)){
					Map<String,List<String>> criteria = new HashMap<String, List<String>>();
					session.setAttribute(CRITERIA, fm.refineSessionCriteria(criteria, locale));
					session.setAttribute(QUICK_CRITERIA, fm.toQuickCriteria(criteria, locale));
				}else{
					// all criteria types which the user set
					String[] criterias = request.getParameterValues(CRITERIA);
					if (criterias == null) { // user don't want to filter anything
						session.removeAttribute(CRITERIA);
						return;
					}
	
					// the amount of criterias the user set
					int criteriaAmount = criterias.length;
					// the user filter
					Map<String,List<String>> criteria = new HashMap<String,List<String>>(criteriaAmount * 2);
					// parse the filter info and store it into the hashmap
					for (int i = 0; i < criteriaAmount; i++) {
						int index = criterias[i].indexOf('_');
						String name = criterias[i].substring(0, index);
						String value = criterias[i].substring(index + 1);
						List<String> temp = criteria.get(name);
						if (temp == null) {
							temp = new ArrayList<String>();
						}
						temp.add(value);
						criteria.put(name, temp);
					}
					// put the filter into session to store it
					criteria = fm.refineSessionCriteria(criteria, locale);
					session.setAttribute(CRITERIA, criteria);
					session.setAttribute(QUICK_CRITERIA, fm.toQuickCriteria(criteria, locale));
				}
			} else if (TRUE.equals(disableFilter)) { // user want to disable the filter
				session.removeAttribute(CRITERIA);
			}
		}

	}

	/**
	 * @return Returns the criteria.
	 */
	public Map<String, List<String>>  getCriteria() {
		return criteria;
	}

	/**
	 * check if a criteria is selected
	 * 
	 * @param key
	 *            the key of the criteria
	 * @param value
	 *            the value of the criteria
	 * @return true if the criteria is already set in the filter
	 */
	public boolean isSelected(String key, String value) {
		boolean selected = false;
		Map<String, List<String>> filter = (Map<String, List<String>>) session.getAttribute(CRITERIA);
		if (filter != null) {
			List<String> temp = filter.get(key);
			if (temp != null) {
				if (temp.contains(value)) {
					selected = true;
				}
			}
		}
		return selected;

	}

	/**
	 * if all items of the criteria group is selected
	 * 
	 * @param key
	 *            the name of the criteria group
	 * @return
	 */
	public boolean isSelected(String key) {
		boolean selected = false;
		Map<String, List<String>> filter = (Map<String, List<String>>) session.getAttribute(CRITERIA);
		if (filter != null) {
			List<String> temp = filter.get(key);
			if (temp != null) {
				selected = temp.containsAll(criteria.get(key));
			} else {
				selected = false;
			}
		}
		return selected;
	}

	/**
	 * if part of the criteria group is selected
	 * 
	 * @param key
	 *            the name of the criteria group
	 * @return
	 */
	public boolean isPartSelected(String key) {
		boolean selected = false;
		Map<String, List<String>> filter = (Map<String, List<String>>) session.getAttribute(CRITERIA);
		if (filter != null) {
			List<String> temp = filter.get(key);
			if (temp != null && temp.size() > 1
					&& temp.size() < criteria.get(key).size() + 1) { //there a an unclassified value now
				selected = true;
			}
		}
		return selected;
	}

	/**
	 * check if the filter is enabled
	 * 
	 * @return true if the filter is enabled
	 */
	public boolean isFilterOn() {
		boolean isOn = false;
		try {
			if (criteria != null) {
				Object temp = session.getAttribute(CRITERIA);
				isOn = !HelpPlugin.getFilterManager().refineSessionCriteria(cloneMap(criteria),locale).equals(temp);
			} else {
				isOn = false;
			}
		} catch (Exception e) { // if there is any run time exception
			isOn = false;
		}

		return isOn;
	}

	/**
	 * get the filter button state
	 * 
	 * @return true if the filter is enabled
	 */
	public String getButtonState() {
		if (cookiEnabled && isFilterEnabled()) {
			return (isFilterOn()) ? ON : OFF;
		} else {
			return HIDDEN;
		}
	}
	
	public boolean isFilterEnabled() {
		return HelpBasePlugin.getDefault().getPluginPreferences().getBoolean("filterEnable");
	}

	/**
	 * the alert of filter button
	 * 
	 * @return
	 */
	public String getButtonToolTip() {
		return (isFilterOn()) ? "filteropen" : "filterclose";
	}

	/**
	 * check if the filter is enabled
	 * 
	 * @param session
	 *            the current user session
	 * @return true if filter is enabled
	 */
	public static boolean isCriteriaFilterEnabled(HttpSession session) {
		return new FilterData(session).isFilterOn();
	}

	/**
	 * check if the topic accord with the filter
	 * 
	 * @param userFilter
	 * @param topic
	 * @return
	 */
	private static boolean filter(Map userFilter, Map topic) {
		return FilterManager.filter(userFilter, topic);
	}

	/**
	 * check if the topic should be filter out
	 * 
	 * @param session
	 * @param topic
	 * @return true if topic fix the filter criteria
	 */
	public static boolean filter(HttpSession session, IFilterableResource topic) {
		if (!FilterData.isCriteriaFilterEnabled(session)) {
			return true;
		}
		return FilterData.filter((Map) session.getAttribute(CRITERIA),
				topic.getCriterias());

	}

	/**
	 * check if the Map topic is fit the filter in session
	 * 
	 * @param session
	 * @param topic
	 * @return
	 */
	public static boolean filter(HttpSession session, Map topic) {
		if (!FilterData.isCriteriaFilterEnabled(session)) {
			return true;
		}
		return FilterData.filter((Map) session.getAttribute(CRITERIA),
				topic);

	}
	
	public static boolean quickFilter(HttpSession session, Map<String,BitSet> quickCriteria) {
		if (!FilterData.isCriteriaFilterEnabled(session)) {
			return true;
		}
		return FilterManager.quickFilter((Map) session.getAttribute(QUICK_CRITERIA),quickCriteria);

	}

	public static Map getFiltersMap(HttpSession session) {
		return (Map) session.getAttribute(CRITERIA);
	}
	
	public static Map<String, BitSet> getQuickFiltersMap(HttpSession session) {
		return (Map<String, BitSet>) session.getAttribute(QUICK_CRITERIA);
	}
	/**
	 * check if the filter is available
	 * 
	 * @return
	 */
	public boolean isFilterAvailable() {
		return criteria.size() > 0 ? true : false;
	}

	/**
	 * clone a map
	 * 
	 * @param map
	 * @return
	 */
	public static Map<String,List<String>> cloneMap(Map map) {
		Map<String,List<String>> temp = new HashMap<String,List<String>>(map.keySet().size() * 2);
		for (Iterator keys = map.keySet().iterator(); keys.hasNext();) {
			Object key = keys.next();
			List<String> v = new ArrayList<String>();
			v.addAll((Collection)map.get(key));
			temp.put((String)key, v);
		}
		return temp;
	}

	/**
	 * disable filter
	 */
	private static final String DISABLE_FILTER = "disableFilter";

	/**
	 * the criteria in session
	 */
	public static final String CRITERIA = "criteria";
	public static final String NOCRITERIA = "nocriteria";
	public static final String QUICK_CRITERIA = "quickCriteria";

	/**
	 * flag false
	 */
	private static final String FALSE = "false";

	/**
	 * flat true
	 */
	private static final String TRUE = "true";

	/**
	 * flag on
	 */
	public static final String ON = "on";

	private static final String HIDDEN = "hidden";

	/**
	 * flag off
	 */
	private static final String OFF = "off";

	/**
	 * @return Returns the cookiEnabled.
	 */
	public boolean isCookiEnabled() {
		return cookiEnabled;
	}

	/**
	 * @param cookiEnabled
	 *            The cookiEnabled to set.
	 */
	public void setCookiEnabled(boolean cookiEnabled) {
		this.cookiEnabled = cookiEnabled;
	}
	
	public boolean isOtherBrowserEnabled() {
		boolean enable = true;
		if(UrlUtil.isSafari(request) && "120".compareTo(UrlUtil.getSafariVersion(request)) <= 0)
			enable = HelpBasePlugin.getDefault().getPluginPreferences().getBoolean("safariEnable");
		return enable;
	}
}
