package org.eclipse.help.internal.webapp.data;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;
import org.eclipse.help.internal.base.HelpBasePlugin;

public class FrameData {
	
	private static IConfigurationElement[] applications;
	
	private static void initialApplications() {
		IExtensionRegistry registry = Platform.getExtensionRegistry();
		IConfigurationElement[] elements = registry
				.getConfigurationElementsFor("org.eclipse.help.webapp.application");
		if (elements == null || elements.length == 0) {
			applications = new IConfigurationElement[0];
			return;
		}
		if (elements[0] == null) {
			applications = new IConfigurationElement[0];
			return;
		}
		applications =  elements;
	}
	
	public static List getCenterApplications() {
		List centerFrameApps = new ArrayList();
		if(applications == null) {
			initialApplications();
		}
		for(int i=0; i<applications.length; i++) {
			if(applications[i].getName().equalsIgnoreCase("centerFrameApp")) {
				centerFrameApps.add(applications[i].getChildren()[0]);
			}
		}
		return centerFrameApps;
	}
	
	public static List getBottomApplications() {
		List bottomFrameApps = new ArrayList();
		if(applications == null) {
			initialApplications();
		}
		for(int i=0; i<applications.length; i++) {
			if(applications[i].getName().equalsIgnoreCase("bottomFrameApp")) {
				bottomFrameApps.add(applications[i].getChildren()[0]);
			}
		}
		return bottomFrameApps;
	}
	
	public static IConfigurationElement getDefaultCenterApplication() {
		
		Iterator centerFrameApps = getCenterApplications().iterator();
		while(centerFrameApps.hasNext()) {
			IConfigurationElement app = (IConfigurationElement)centerFrameApps.next();
			if((app).getAttribute("default").equals("true")){
				return app;
			}
		}
		
		return null;
		
	}
	
	public static IConfigurationElement getDefaultBottomApplication() {
		
		Iterator bottomFrameApps = getBottomApplications().iterator();
		while(bottomFrameApps.hasNext()) {
			IConfigurationElement app = (IConfigurationElement)bottomFrameApps.next();
			if((app).getAttribute("default").equals("true")){
				return app;
			}
		}
		
		return null;
		
	}
	
	public static boolean hasDefaultCenterApplication() {
		Iterator centerFrameApps = getCenterApplications().iterator();
		while(centerFrameApps.hasNext()) {
			IConfigurationElement app = (IConfigurationElement)centerFrameApps.next();
			if((app).getAttribute("default").equals("true")){
				return true;
			}
		}
		
		return false;
	}
	
	public static boolean hasDefaultBottomApplication() {
		Iterator bottomFrameApps = getBottomApplications().iterator();
		while(bottomFrameApps.hasNext()) {
			IConfigurationElement app = (IConfigurationElement)bottomFrameApps.next();
			if((app).getAttribute("default").equals("true")){
				return true;
			}
		}
		return false;
	}
	
	public static String getInfocenterContext() {
		String infocenterContext = HelpBasePlugin.getDefault().getPluginPreferences().getString("infocenterContext");
		if(infocenterContext.equals("/")){
			infocenterContext = "";
		}
		return infocenterContext;
	}

}
