/*******************************************************************************
 * Copyright (c) 2000, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     Sebastian Davids <sdavids@gmx.de> - fix for Bug 182466 
 *     2008/08/08 - updated by Hu Qiongkai(IBM Corp.) for Story345 Quick Search
 *     2008/08/18 - updated by Xu Yumei(IBM Corp.) for Group Result
 *******************************************************************************/
package org.eclipse.help.internal.webapp.data;

import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.servlet.ServletContext;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.eclipse.help.HelpSystem;
import org.eclipse.help.IHelpResource;
import org.eclipse.help.IToc;
import org.eclipse.help.ITopic;
import org.eclipse.help.internal.HelpPlugin;
import org.eclipse.help.internal.base.BaseHelpSystem;
import org.eclipse.help.internal.base.HelpBasePlugin;
import org.eclipse.help.internal.search.ISearchQuery;
import org.eclipse.help.internal.search.QueryTooComplexException;
import org.eclipse.help.internal.search.SearchHit;
import org.eclipse.help.internal.search.SearchProgressMonitor;
import org.eclipse.help.internal.search.SearchQuery;
import org.eclipse.help.internal.search.SearchResults;
import org.eclipse.help.internal.toc.FilterManager;
import org.eclipse.help.internal.webapp.HelpWebappPlugin;
import org.eclipse.help.internal.webapp.servlet.WebappWorkingSetManager;
import org.eclipse.help.internal.workingset.AdaptableHelpResource;
import org.eclipse.help.internal.workingset.AdaptableToc;
import org.eclipse.help.internal.workingset.WorkingSet;
import org.eclipse.help.search.ISearchEngineResult;
import org.eclipse.help.search.ISearchEngineResult2;

/**
 * Helper class for searchView.jsp initialization
 */
public class SearchData extends ActivitiesData {
	private static final String PERCENTAGE_DISPLAY = "percentageDisplay";

	private static boolean percentageDisplay = HelpBasePlugin.getDefault()
			.getPluginPreferences().getBoolean(PERCENTAGE_DISPLAY);;

	private WebappWorkingSetManager wsmgr;

	// Request parameters
	private String topicHref;

	private String selectedTopicId = ""; //$NON-NLS-1$

	private String searchWord;

	private String workingSetName;

	// search results
	SearchHit[] hits;

	private boolean hasInitCriteria = false;

	// quick search start point topic id
	private String quickSearchTopicID;

	// quick search start point topic href
	private String quickSearchTopicHref;

	// percentage of indexing completion
	private int indexCompletion = 100;

	// QueryException if any
	private QueryTooComplexException queryException = null;

	/**
	 * Constructs the xml data for the search results page.
	 * 
	 * @param context
	 * @param request
	 */
	public SearchData(ServletContext context, HttpServletRequest request,
			HttpServletResponse response) {
		super(context, request, response);

		wsmgr = new WebappWorkingSetManager(request, response, getLocale());
		this.topicHref = request.getParameter("topic"); //$NON-NLS-1$
		if (topicHref != null && topicHref.length() == 0)
			topicHref = null;

		searchWord = request.getParameter("searchWord"); //$NON-NLS-1$ 

		// group search
		Map criterias = HelpPlugin.getFilterManager().getCriterias(getLocale());
		if (null != criterias) {
			groupTypes = HelpPlugin.getFilterManager()
					.getCriterias(getLocale()).keySet().iterator();
		} else
			groupTypes = new ArrayList(0).iterator();
		// group search ends

		if (isQuickSearchRequest()) {
			quickSearchTopicID = request.getParameter("quickSearchTopicID");

			if (searchWord != null) {
				request.getSession()
						.setAttribute("quickSearchWord", searchWord);
			}
		}

		// try loading search results or get the indexing progress info.
		if (isSearchRequest() && !isScopeRequest()) {
			loadSearchResults();
			if (queryException != null) {
				return;
			}
			if (!isProgressRequest()) {
				for (int i = 0; i < hits.length; i++) {
					if (hits[i].getHref().equals(topicHref)) {
						selectedTopicId = "a" + i; //$NON-NLS-1$
						break;
					}
				}
			}

		}

		// group search

		// group search ends
	}

	// group search
	public boolean isPercentageDisplay() {
		return percentageDisplay;
	}

	public String getTopicScore(int i) {
		try {
			float score = hits[i].getScore();
			NumberFormat percentFormat = NumberFormat
					.getPercentInstance(UrlUtil.getLocaleObj(request, response));
			return percentFormat.format(score);
		} catch (NumberFormatException nfe) {
			// will display original score string
			return String.valueOf(hits[i].getScore());
		}
	}

	public String getTopicScore(SearchHit hit) {
		try {
			float score = hit.getScore();
			NumberFormat percentFormat = NumberFormat
					.getPercentInstance(UrlUtil.getLocaleObj(request, response));
			return percentFormat.format(score);
		} catch (NumberFormatException nfe) {
			// will display original score string
			return String.valueOf(hit.getScore());
		}
	}

	private void groupSearchResults() {
//		initCriteria();
		String Uncategorized = ServletResources.getString("Uncategorized",
				request);
		int len = hits.length;
		results = new HashMap();
		for (int i = 0; i < len; i++) {
			Set criteriaValues = (Set) hits[i].getCriteria().get(group);
			if (criteriaValues != null) {
				for (Iterator values = criteriaValues.iterator(); values
						.hasNext();) {
					String value = (String) values.next();
					if(FilterManager.UNCLASSIFIED.equals(value)){
						value = Uncategorized;
					}
					List hitsGroup = (List) results.get(value);
					if (hitsGroup == null) {
						hitsGroup = new ArrayList();
					}
					hitsGroup.add(hits[i]);
					results.put(value, hitsGroup);
				}
			} else {
				List hitsGroup = (List) results.get(Uncategorized);
				if (hitsGroup == null) {
					hitsGroup = new ArrayList();
				}
				hitsGroup.add(hits[i]);
				results.put(Uncategorized, hitsGroup);
			}

		}
	}

	// group search ends

	private boolean isGroupSearch(){
		group = request.getParameter("group");
		return group != null && !group.equalsIgnoreCase("none");
	}
	
	/**
	 * Returns true when there is a quick search request
	 * 
	 * @return boolean
	 */
	public boolean isQuickSearchRequest() {
		return isSingleTopicSearch() || isSubTopicSearch();
	}

	/**
	 * Returns true when there is a subtopics quick search request
	 * 
	 * @return boolean
	 */
	public boolean isSubTopicSearch() {
		return (request.getParameter("subTopicsSearch") != null); //$NON-NLS-1$
	}

	/**
	 * Returns true when there is a search request
	 * 
	 * @return boolean
	 */
	public boolean isSearchRequest() {
		return (request.getParameter("searchWord") != null); //$NON-NLS-1$
	}

	/**
	 * Return indexed completion percentage
	 */
	public boolean isProgressRequest() {
		return (hits == null && indexCompletion != 100);
	}

	/**
	 * Returns true when there is a request to change the scope (working set)
	 */
	public boolean isScopeRequest() {
		return (request.getParameter("workingSet") != null); //$NON-NLS-1$
	}

	/**
	 * Returns true when there is a single topic quick search request
	 * 
	 * @return boolean
	 */
	public boolean isSingleTopicSearch() {
		return (request.getParameter("singleTopicSearch") != null); //$NON-NLS-1$
	}

	/**
	 * Returns the quick search query
	 */
	public String getQuickSearchWord() {
		if (request.getSession().getAttribute("quickSearchWord") == null)
			return ""; //$NON-NLS-1$
		return request.getSession().getAttribute("quickSearchWord").toString();
	}

	/**
	 * Returns the quick search topic ID
	 */
	public String getQuickSearchTopicID() {
		if (quickSearchTopicID == null)
			return ""; //$NON-NLS-1$
		return quickSearchTopicID;
	}

	private HashMap results;

	public HashMap getResults() {
		return results;
	}

	public String getCategoryLabel(int i) {
		IHelpResource cat = hits[i].getCategory();
		if (cat != null) {
			return cat.getLabel();
		}
		return null;
	}

	public String getCategoryHref(int i) {
		IHelpResource cat = hits[i].getCategory();
		if (cat != null) {
			String tocHref = cat.getHref();
			IToc[] tocs = HelpSystem.getTocs();
			for (int j = 0; j < tocs.length; ++j) {
				if (tocHref.equals(tocs[j].getHref())) {
					ITopic topic = tocs[j].getTopic(null);
					String topicHref = topic.getHref();
					if (topicHref != null) {
						return UrlUtil.getHelpURL(topicHref);
					}
					return "../nav/" + j; //$NON-NLS-1$
				}
			}
		}
		return null;
	}

	/**
	 * Return the number of links
	 * 
	 * @return int
	 */
	public int getResultsCount() {
		return hits.length;
	}

	public String getSelectedTopicId() {
		return selectedTopicId;
	}

	public String getTopicHref(int i) {
		return UrlUtil.getHelpURL(hits[i].getHref());
	}

	public String getTopicLabel(int i) {
		return UrlUtil.htmlEncode(hits[i].getLabel());
	}

	public String getTopicTocLabel(int i) {
		if (hits[i].getToc() != null)
			return UrlUtil.htmlEncode(hits[i].getToc().getLabel());
		return ""; //$NON-NLS-1$
	}

	public String getTopicDescription(int i) {
		String description = hits[i].getDescription();
		if (description != null) {
			return UrlUtil.htmlEncode(description);
		}
		return ""; //$NON-NLS-1$
	}

	/**
	 * @param i
	 * @return true of result belong to an enabled TOC
	 */
	public boolean isEnabled(int i) {
		String href = hits[i].getHref();
		return HelpBasePlugin.getActivitySupport().isEnabledTopic(href,
				getLocale());
	}

	// search group
	public boolean isFiltered(int i) {
		try {
			if (!this.isSubTopicSearch()) {
				return false;
			}
			return !FilterData.quickFilter(request.getSession(), hits[i].getQuickCriteria());
		} catch (Exception e) {
			return true;
		}
	}

	public boolean isFilterEnabled() {
		return HelpBasePlugin.getDefault().getPluginPreferences().getBoolean(
				"filterEnable");
	}

	// search group ends

	/**
	 * Returns whether or not the ith hit is a potential hit. This means it may
	 * not be an actual hit (i.e. it found something in a filtered section of
	 * the document).
	 * 
	 * @param i
	 *            the index of the hit to check
	 * @return whether or not the hit is a potential hit
	 */
	public boolean isPotentialHit(int i) {
		return ((getMode() != MODE_INFOCENTER) && hits[i].isPotentialHit());
	}

	public boolean isShowCategoriesButtonOn() {
		if (isShowCategories())
			return true;
		else if (!getCookieValue("groupBy").equals("None")
				&& !getCookieValue("groupBy").equals("false")) {
			return true;
		}

		return false;
	}

	public boolean isShowCategories() {
		// default off
		return String.valueOf(true).equals(getCookieValue("showCategories"));
	}

	public boolean isGroupBy() {
		// default off
		return String.valueOf(true).equals(getCookieValue("groupBy"));
	}

	public boolean isShowDescriptions() {
		// default on
		return String.valueOf(true).equals(getCookieValue("showDescriptions"));
	}

	public String getCookieValue(String name) {
		Cookie[] cookies = request.getCookies();
		if (cookies != null) {
			for (int i = 0; i < cookies.length; ++i) {
				if (name.equals(cookies[i].getName())) { //$NON-NLS-1$
					return cookies[i].getValue();
				}
			}
		}

		if (name.equals("showDescriptions"))
			return "true"; // default on
		else
			// default off
			return "false";
	}

	/**
	 * Return indexed completion percentage
	 */
	public String getIndexedPercentage() {
		return String.valueOf(indexCompletion);
	}

	/**
	 * Returns the search query
	 */
	public String getSearchWord() {
		if (searchWord == null)
			return ""; //$NON-NLS-1$
		return searchWord;
	}

	/**
	 * Returns the list of selected TOC's
	 */
	public String[] getSelectedTocs() {
		String[] books = request.getParameterValues("scope"); //$NON-NLS-1$
		if (books == null) {
			// select all books
			TocData tocData = new TocData(context, request, response);
			books = new String[tocData.getTocCount()];
			for (int i = 0; i < books.length; i++)
				books[i] = tocData.getTocHref(i);
		}
		return books;
	}

	/**
	 * Returns true if book is within a search scope
	 */
	public boolean isTocSelected(int toc) {
		TocData tocData = new TocData(context, request, response);
		String href = tocData.getTocHref(toc);
		String[] books = request.getParameterValues("scope"); //$NON-NLS-1$
		if (books == null)
			return false;
		for (int i = 0; i < books.length; i++) {
			if (books[i].equals(href)) {
				return true;
			}
		}
		return false;
	}

	/**
	 * Returns the working set selected. This is used to display the working set
	 * name in the search banner.
	 * 
	 * @return String
	 */
	public String getScope() {
		if (workingSetName != null)
			return workingSetName;

		if (isScopeRequest()) {
			workingSetName = request.getParameter("workingSet"); //$NON-NLS-1$
			saveWorkingSet(workingSetName);
		} else if (isSearchRequest()) {
			workingSetName = request.getParameter("scope"); //$NON-NLS-1$
			// if we have already set the working set, then use it.
			if (workingSetName == null) {
				workingSetName = request.getParameter("workingSet"); //$NON-NLS-1$
			}
		} else {
			workingSetName = wsmgr.getCurrentWorkingSet();
		}

		if (workingSetName == null || workingSetName.length() == 0
				|| getMode() == RequestData.MODE_INFOCENTER
				&& wsmgr.getWorkingSet(workingSetName) == null)
			workingSetName = ServletResources.getString("All", request); //$NON-NLS-1$
		return workingSetName;
	}

	/**
	 * This method is used to persist the working set name and is called from
	 * the search view, after each search
	 */
	public void saveScope() {
		// if a working set is defined, set it in the preferences
		String workingSet = request.getParameter("scope"); //$NON-NLS-1$
		saveWorkingSet(workingSet);
	}

	private void saveWorkingSet(String workingSet) {
		String lastWS = wsmgr.getCurrentWorkingSet();
		if (workingSet != null && !workingSet.equals(lastWS)) {
			wsmgr.setCurrentWorkingSet(workingSet);
		} else if (workingSet == null && lastWS != null && lastWS.length() > 0) {
			wsmgr.setCurrentWorkingSet(""); //$NON-NLS-1$
		}
	}

	/**
	 * Call the search engine, and get results or the percentage of indexed
	 * documents.
	 */
	private void loadSearchResults() {
		try {
			SearchProgressMonitor pm = SearchProgressMonitor
					.getProgressMonitor(getLocale());
			if (pm.isDone()) {
				this.indexCompletion = 100;
				SearchResults results = createHitCollector();
				BaseHelpSystem.getSearchManager().search(createSearchQuery(),
						results, pm);
				hits = results.getSearchHits();
				if (hits == null) {
					HelpWebappPlugin
							.logWarning("No search results returned.  Help index is in use."); //$NON-NLS-1$
				} else {
//					if (this.isSubTopicSearch()) {
//						this.initCriteria();
//						this.filterResults();
//					}
					if (isShowCategories()) {
						Arrays.sort(hits, new SearchResultComparator());
					} else {
						// group search starts
						if (isGroupSearch() && hits != null) {
							groupSearchResults();
						}
					}
					// group search ends
				}
				return;
			}
			// progress
			indexCompletion = pm.getPercentage();
			if (indexCompletion >= 100) {
				// 38573 We do not have results, so index cannot be 100
				indexCompletion = 100 - 1;
			}
			return;
		} catch (QueryTooComplexException qe) {
			queryException = qe;
		} catch (Exception e) {
			this.indexCompletion = 0;
			e.printStackTrace();
		}

	}

	private ISearchQuery createSearchQuery() {
		String fieldSearchStr = request.getParameter("fieldSearch"); //$NON-NLS-1$
		boolean fieldSearch = fieldSearchStr != null ? new Boolean(
				fieldSearchStr).booleanValue() : false;
		SearchQuery sq = new SearchQuery(
				searchWord == null ? "" : searchWord, fieldSearch, new ArrayList(), //$NON-NLS-1$
				getLocale());
		if(isSingleTopicSearch()){
			// remove the anchor part, because the href in search index has not
			// this part.
			String url = quickSearchTopicHref;
			int index = quickSearchTopicHref.indexOf("#");
			if(index != -1){
				url = quickSearchTopicHref.substring(0, index);
			}
			sq.setUrl(url);
		}
		return sq;
	}

	private SearchResults createHitCollector() {
		WorkingSet[] workingSets;

		int maxHits = 500;
		String maxHitsStr = request.getParameter("maxHits"); //$NON-NLS-1$
		if (maxHitsStr != null) {
			try {
				int clientmaxHits = Integer.parseInt(maxHitsStr);
				if (0 < clientmaxHits && clientmaxHits < 500) {
					maxHits = clientmaxHits;
				}
			} catch (NumberFormatException nfe) {
			}
		}

		if (isQuickSearchRequest()) {
			workingSets = createQuickWorkingSetByID(quickSearchTopicID);
			if (isSingleTopicSearch()) {
				return new SearchResults(workingSets, 
						maxHits, 
						getLocale(), 
						false, 
						true, 
						FilterData.isCriteriaFilterEnabled(request.getSession()),
						quickSearchTopicHref,
						isGroupSearch());
			}
			return new SearchResults(workingSets, 
					maxHits, 
					getLocale(), 
					true, 
					false, 
					FilterData.isCriteriaFilterEnabled(request.getSession()), 
					null, 
					FilterData.getQuickFiltersMap(request.getSession()),
					isGroupSearch());
		}

		else if (request.getParameterValues("scopedSearch") == null) { //$NON-NLS-1$
			// scopes are working set names
			workingSets = getWorkingSets();
			if(request.getParameter("scope") == null) {
				return new SearchResultFilter(workingSets, maxHits, getLocale(), false, isGroupSearch());
			} else {
				return new SearchResultFilter(workingSets, maxHits, getLocale(), true, isGroupSearch());
			}
		} else {
			// scopes are books (advanced search)
			workingSets = createTempWorkingSets();
			return new SearchResultFilter(workingSets, maxHits, getLocale(), isGroupSearch());
		}
	}

	/**
	 * @return WorkingSet[] or null
	 */
	private WorkingSet[] getWorkingSets() {
		String[] scopes = request.getParameterValues("scope"); //$NON-NLS-1$
		if (scopes == null) {
			return null;
		}
		// confirm working set exists and use it
		ArrayList workingSetCol = new ArrayList(scopes.length);
		for (int s = 0; s < scopes.length; s++) {
			WorkingSet ws = wsmgr.getWorkingSet(scopes[s]);
			if (ws != null) {
				workingSetCol.add(ws);
			}
		}
		if (workingSetCol.size() == 0) {
			return null;
		}
		return (WorkingSet[]) workingSetCol
				.toArray(new WorkingSet[workingSetCol.size()]);
	}

	/**
	 * @return WorkingSet[] or null
	 */
	private WorkingSet[] createTempWorkingSets() {
		String[] scopes = request.getParameterValues("scope"); //$NON-NLS-1$
		if (scopes == null) {
			// it is possible that filtering is used, but all books are
			// deselected
			return new WorkingSet[0];
		}
		if (scopes.length == HelpPlugin.getTocManager().getTocs(getLocale()).length) {
			// do not filter if all books are selected
			return null;
		}
		// create working set from books
		ArrayList tocs = new ArrayList(scopes.length);
		for (int s = 0; s < scopes.length; s++) {
			AdaptableToc toc = wsmgr.getAdaptableToc(scopes[s]);
			if (toc != null) {
				tocs.add(toc);
			}
		}
		AdaptableToc[] adaptableTocs = (AdaptableToc[]) tocs
				.toArray(new AdaptableToc[tocs.size()]);
		WorkingSet[] workingSets = new WorkingSet[1];
		workingSets[0] = wsmgr.createWorkingSet("temp", adaptableTocs); //$NON-NLS-1$
		return workingSets;
	}

	public String getQueryExceptionMessage() {
		if (queryException == null) {
			return null;
		}
		return ServletResources.getString("searchTooComplex", request); //$NON-NLS-1$
	}

	/*
	 * Filters out results that help doesn't know how to open (i.e. those hits
	 * that implement ISearchEngineResult2 and canOpen() returns true.
	 */
	private static class SearchResultFilter extends SearchResults {
		public SearchResultFilter(WorkingSet[] workingSets, int maxHits,
				String locale, boolean isGroupSearch) {
			super(workingSets, maxHits, locale, isGroupSearch);
		}
		public SearchResultFilter(WorkingSet[] workingSets, int maxHits,
				String locale, boolean isScopeSearch,boolean isGroupSearch) {
			super(workingSets, maxHits, locale, isScopeSearch, isGroupSearch);
		}

		public void addHits(List hits, String highlightTerms) {
			List filtered = new ArrayList();
			Iterator iter = hits.iterator();
			while (iter.hasNext()) {
				Object obj = iter.next();
				if (!(obj instanceof ISearchEngineResult2 && ((ISearchEngineResult2) obj)
						.canOpen())) {
					filtered.add(obj);
				}
			}
			super.addHits(filtered, highlightTerms);
		}
	}

	private static class SearchResultComparator implements Comparator {
		public int category(Object element) {
			if (element instanceof ISearchEngineResult) {
				ISearchEngineResult r = (ISearchEngineResult) element;
				IHelpResource c = r.getCategory();
				if (c != null) {
					String label = c.getLabel();
					if (label.length() == 0)
						return 10;
					return 5;
				}
			}
			return 0;
		}

		public int compare(Object e1, Object e2) {
			int cat1 = category(e1);
			int cat2 = category(e2);
			if (cat1 != cat2) {
				return cat1 - cat2;
			}
			ISearchEngineResult r1 = (ISearchEngineResult) e1;
			ISearchEngineResult r2 = (ISearchEngineResult) e2;
			IHelpResource c1 = r1.getCategory();
			IHelpResource c2 = r2.getCategory();
			if (c1 != null && c2 != null) {
				int cat = c1.getLabel().compareToIgnoreCase(c2.getLabel());
				if (cat != 0) {
					return cat;
				}
			}
			float rank1 = ((ISearchEngineResult) e1).getScore();
			float rank2 = ((ISearchEngineResult) e2).getScore();
			if (rank1 - rank2 > 0) {
				return -1;
			} else if (rank1 == rank2) {
				return 0;
			} else {
				return 1;
			}
		}
	}

	/**
	 * 
	 * @return WorkingSet[] or null
	 */
	private WorkingSet[] createQuickWorkingSetByID(String quickSearchTopicID) {

		ArrayList adaptableHelpResource = new ArrayList(1);

		if (quickSearchTopicID != null) {

			AdaptableHelpResource topic = null;
			IToc[] tocs = HelpPlugin.getTocManager().getTocs(getLocale());
			// uat4i00000613
			int indexOf_ = quickSearchTopicID.indexOf("_");
			int tocid = 0;
			if (indexOf_ != -1) {

				tocid = Integer.parseInt(quickSearchTopicID.substring(1,
						indexOf_));
			} else {

				tocid = Integer.parseInt(quickSearchTopicID.substring(1,
						quickSearchTopicID.length()));
			}
			// int tocid= Integer.parseInt(quickSearchTopicID.substring(1, 2));

			// end of uat4i00000613
			IToc toc = tocs[tocid];
			AdaptableHelpResource adaptableToc = wsmgr.getAdaptableToc(toc
					.getHref());
			// if quickSearchTopicID start with 'b' , the id is from a book node
			if (quickSearchTopicID.charAt(0) == 'b') {

				this.quickSearchTopicHref = adaptableToc.getTopic(null)
						.getHref();

				adaptableHelpResource.add(adaptableToc);

			}
			// if quickSearchTopicID start with 't' , the id is from a topic
			// node
			else if (quickSearchTopicID.charAt(0) == 't')

			{
				String[] paths = quickSearchTopicID.split("_");

				// indexes should not contain the book id part.
				int[] indexes = new int[paths.length - 1];
				boolean indexOK = true;

				for (int i = 0; i < indexes.length; i++) {
					try {
						indexes[i] = Integer.parseInt(paths[i + 1]);
					} catch (NumberFormatException nfe) {
						indexOK = false;
						break;

					}
				}

				if (indexOK) {
					AdaptableHelpResource[] topics = adaptableToc.getChildren();

					for (int p = 0; p < indexes.length; p++) {
						// this topic is where user start the quick search.
						topic = topics[indexes[p]];
						topics = topic.getChildren();
					}
					this.quickSearchTopicHref = topic.getHref();
					adaptableHelpResource.add(topic);
				}

			}

			else {

				return null;
			}

			// navigate to root topic, skipping parents

		}

		WorkingSet[] workingSets = new WorkingSet[1];

		workingSets[0] = new WorkingSet("QuickSearch", adaptableHelpResource);

		return workingSets;
	}

	private String group = "none";

	private String detail;

	private Iterator groupTypes;

	public String getGroup() {
		return group;
	}

	public void setGroup(String group) {
		this.group = group;
	}

	public String getDetail() {
		return detail;
	}

	public Iterator getGroupTypes() {
		return groupTypes;
	}

	// group search
//	private void initCriteria() {
//		if (this.hasInitCriteria)
//			return;
//		else {
//			if (hits != null) {
//				int len = hits.length;
//				for (int i = 0; i < len; i++) {
//					IToc toc = hits[i].getToc();
//					String href = hits[i].getHref().substring(0,
//							hits[i].getHref().indexOf("?"));
//					Map criteria = Collections.emptyMap();
//					Map<String,BitSet> quickCriteria = Collections.emptyMap();
//					if (toc != null) {
//						ITopic temp = toc.getTopic(href);
//						if (temp != null) {
//							criteria = temp.getCriteria();
//							quickCriteria = temp.getQuickCriteria(getLocale());
//						} else {
//							HelpWebappPlugin.logError(
//									"invalid topic href in search grouping",
//									null/* , new NullPointerException() */);
//						}
//					}
//					hits[i].setCriteria(criteria, quickCriteria);
//				}
//				this.hasInitCriteria = true;
//			}
//		}
//	}

	public void filterResults() {
		List tempResults = new ArrayList();
		int len = hits.length;
		for (int i = 0; i < len; i++) {

			if (FilterData.quickFilter(request.getSession(), hits[i].getQuickCriteria()))
				tempResults.add(hits[i]);
		}
		int resultLength = tempResults.size();
		hits = new SearchHit[resultLength];
		for (int i = 0; i < resultLength; i++) {
			hits[i] = (SearchHit) tempResults.get(i);
		}

	}

	// group search ends

	// Help to diplay the right tip for button according its state, added by WHZ
	// 2008/09/08
	/**
	 * the tip of show description button
	 * 
	 * @return
	 */
	public String getButtonToolTip() {
		return (isShowDescriptions()) ? "show_descriptions_off"
				: "show_descriptions_on";
	}

	public String getGroupButtonTip() {
		return (isShowCategoriesButtonOn()) ? "group_search_result_off"
				: "group_search_result_on";
	}
}
