package org.eclipse.help.internal.webapp.servlet;

import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;
import org.eclipse.help.internal.webapp.HelpWebappPlugin;

/**
 * @author May
 */
public class ExtraFilters implements IFilter {

	public static final String CONTENTFILTER_XP_NAME = "org.eclipse.help.webapp.contentFilter";

	private List<IPrioritizedFilter> filters = new ArrayList<IPrioritizedFilter>();

	public ExtraFilters() {
		IExtensionRegistry registry = Platform.getExtensionRegistry();
		IConfigurationElement[] elements = registry
				.getConfigurationElementsFor(CONTENTFILTER_XP_NAME);
		for (int i = 0; i < elements.length; i++) {

			Object obj = null;
			try {
				obj = elements[i].createExecutableExtension("class");
			} catch (CoreException e) {
				HelpWebappPlugin.logError("Create extension failed:["
						+ CONTENTFILTER_XP_NAME + "].", e);
			}
			if (obj instanceof IPrioritizedFilter) {
				IPrioritizedFilter filter = (IPrioritizedFilter) obj;

				int priority = 0;
				String priStr = elements[i].getAttribute("priority");
				if (priStr != null && !"".equals(priStr)) {
					try {
						priority = Integer.parseInt(priStr);
					} catch (NumberFormatException e) {
					}
				}
				filter.setPriority(priority);
				filters.add(filter);
			}
		}
		Collections.sort(filters);
	}

	public OutputStream filter(HttpServletRequest req, OutputStream out) {
		for (IPrioritizedFilter filter : filters) {
			out = filter.filter(req, out);
		}
		return out;
	}

}
