package org.eclipse.help.internal.webapp.servlet;

import java.io.ByteArrayOutputStream;
import java.io.FilterOutputStream;
import java.io.IOException;
import java.io.OutputStream;

import javax.servlet.http.HttpServletRequest;

public class HTMLEmptyTagFilter implements IFilter{

	public OutputStream filter(HttpServletRequest req, OutputStream out) {
		String uri = req.getRequestURI();
		if (uri == null || !uri.endsWith("html") && !uri.endsWith("htm")) { //$NON-NLS-1$ //$NON-NLS-2$
			return out;
		}
		if ("/rtopic".equals(req.getServletPath())) { //$NON-NLS-1$
			return out;
		}
		
		return new HTMLEmptyTagOutputStream(out);
	}

	/**
	 * Add &lt;!--//leave me alone--&gt; into empty html tags.
	 * Such as &lt;script ...&gt;&lt;/script&gt;
	 * 
	 * Will not add comment into empty tags inside comment.
	 * 
	 * @author BlueGuitar
	 *
	 */
	static class HTMLEmptyTagOutputStream extends FilterOutputStream {
		//></
		private static final int STATE_START = 0;
		private static final int STATE_GT = 1;
		private static final int STATE_LT = 2;
		private static final int STATE_LT_SLASH = 3;

		private int state = STATE_START;

		private byte[] comment = new byte[]{'<','!','-','-','/','/','l','e','a','v','e',' ','m','e',' ','a','l','o','n','e','-','-','>'};


		
		public HTMLEmptyTagOutputStream(OutputStream out) {
			super(out);
		}

		public void write(int b) throws IOException {
			//check is now inside a comment
			//comment do not support wrapping
			checkInComment(b);
			saveLastTag(b);
			switch (state) {
			case STATE_START:
				if (b == '>') {
					state = STATE_GT;
				}
				out.write(b);
				break;
			case STATE_GT:
				if (b == '<') {
					state = STATE_LT;
				} else {
					out.write(b);
					state = STATE_START;
				}
				break;
			case STATE_LT:
				if (b == '/' && null != lastTag) {
					state = STATE_LT_SLASH;
					lastTagIndex = 0;
				}else{
					state = STATE_START;
					out.write('<');
					out.write(b);
				}
				break;
			case STATE_LT_SLASH:
				if(lastTagIndex < lastTag.length ){
					if(Character.toLowerCase((char) b) == Character.toLowerCase((char) lastTag[lastTagIndex])){
						//match
						++lastTagIndex;
					}else{
						//not match
						out.write('<');
						out.write('/');
						for(int i=0;i<lastTagIndex;++i){
							out.write(lastTag[i]);
						}
						out.write(b);
						state = STATE_START;
					}
				}else if(lastTagIndex == lastTag.length){
					//match head
					if(b == '>' || b == ' ' || b == '\t' || b == '\n' || b == '\f'|| b == '\r' || b == 0x0B){
						//match
						//insert comment
						if(!inComment){ 
							out.write(comment);
						}
					}
					out.write('<');
					out.write('/');
					//write back
					for(int i=0;i<lastTagIndex;++i){
						out.write(lastTag[i]);
					}
					out.write(b);
					lastTag = null;
					state = STATE_START;
				}
				break;
			default:
				out.write(b);
				break;
			}
		}

		
		private static final int LAST_STATE_START = 0;
		private static final int LAST_STATE_LT = 1;

		private int lastTagState = STATE_START;
		private ByteArrayOutputStream buf = new ByteArrayOutputStream();
		private byte[] lastTag = null;
		private int lastTagIndex = 0;
		private void saveLastTag(int b) {
			switch(lastTagState){
			case LAST_STATE_START:
				if(b == '<'){
					lastTagState = LAST_STATE_LT;
				}
				break;
			case LAST_STATE_LT:
				if(b == '/'){
					//matched tag
					lastTagState = LAST_STATE_START;
					buf.reset();
				}else if(b == '>' || b == ' ' || b == '\t' || b == '\n' || b == '\f'|| b == '\r' || b == 0x0B){
					//end tag or it is whitespace
					lastTagState = LAST_STATE_START;
					lastTag = buf.toByteArray();
					buf.reset();
				}else{
					buf.write(b);
				}
				break;
			 
			}
		}

		
		private boolean inComment = false;
		//<!-- -->
		private static final int COMMENT_START = 0;
		private static final int COMMENT_GT = 1;
		private static final int COMMENT_GT1 = 2;
		private static final int COMMENT_GT1_ = 3;
		private static final int COMMENT_GT1__ = 4;
		
		private static final int COMMENT_GT1___ = 5;
		private static final int COMMENT_GT1____ = 6;
		
		private int comment_state = COMMENT_START;
		private void checkInComment(int b) {
			if(!inComment){
				switch (comment_state) {
				case COMMENT_START:
					if (b == '<') {
						comment_state = COMMENT_GT;
					}
					break;
				case COMMENT_GT:
					if (b == '!') {
						comment_state = COMMENT_GT1;
					} else {
						comment_state = COMMENT_START;
					}
					break;
				case COMMENT_GT1:
					if (b == '-') {
						comment_state = COMMENT_GT1_;
					} else {
						comment_state = COMMENT_START;
					}
					break;
				case COMMENT_GT1_:
					if (b == '-') {
						comment_state = COMMENT_GT1__;
						inComment = true;
					} else {
						comment_state = COMMENT_START;
					}
					break;
				}
			}else{
				switch (comment_state) {
				case COMMENT_GT1__:
					if (b == '-') {
						comment_state = COMMENT_GT1___;
					}
					break;
				case COMMENT_GT1___:
					if (b == '-') {
						comment_state = COMMENT_GT1____;
					} else {
						comment_state = COMMENT_GT1__;
					}
					break;
				case COMMENT_GT1____:
					if (b == '>') {
						inComment = false;
					}
					comment_state = COMMENT_START;
					break;
				}
			}
		}
	}
	
	public static void main(String[] args){
		OutputStream os = new HTMLEmptyTagFilter().filter(null, System.out);
		try {
//			os.write("<a><B></b></a> <!-- <s></s> -->".getBytes());
			os.write("<script src=\"../com.ibm.srcf.doc/srcfv2_ids1150.js\" type=\"text/javascript\" xml:space=\"preserve\" ></script>".getBytes());
			os.flush();
			os.close();
			//expected result: <a><b><!--//leave me alone--></b></a> <!-- <s></s> -->
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}
